package img.connection;

import java.io.*;
import java.nio.channels.*;

/***********************************************************************
 * Defines methods for some type of I/O Connection.
 */
public interface Connection {

   /**
    * Connects this connection.
    * @throws IOException If the connection could not be connected.
    */
   public void connect() throws Exception;

   /**
    * Disconnects this connection.
    * @throws IOException If the connection could not be disconnected.
    */
   public void disconnect() throws IOException;

   /**
    * Returns the Object to which this connection is wrapped around.
    * @return The Object to which this connection is wrapped around.
    */
   public Object getConnectionObject();

   /**
    * Tells whether or not this connection is connected.
    * @return true if, and only if, this connection is connected.
    */
   public boolean isConnected();

   /**
    * Returns the InputStream associated with this connection.
    * @return The InputStream associated with this connection.
    * @throws IOException If the InputStream could not be returned.
    */
   public InputStream getInputStream() throws IOException;

   /**
    * Returns the OutputStream associated with this connection.
    * @return The OutputStream associated with this connection.
    * @throws IOException If the OutputStream could not be returned.
    */
   public OutputStream getOutputStream() throws IOException;

   /**
    * Returns the ByteChannel associated with this connection.
    * @return The ByteChannel associated with this connection.
    */
   public ByteChannel getChannel();

}
