package img.swing;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;

import img.device.event.*;

import img.lang.*;

public class JAboutDialog extends JDialog {

   private static final String ABOUT_INFO =
      "AVR Imaging System:<br>" +
      "&nbsp;&nbsp;&nbsp;Version: " + ImageSystem.RES.getString("build.major") + "." + ImageSystem.RES.getString("build.minor") + "<br>" +
      "&nbsp;&nbsp;&nbsp;Build Date: " + ImageSystem.RES.getString("build.date");

   private JLabel aboutL;
   private DataListener handler;
   private JMessagePanel messageP;

   public JAboutDialog(Frame owner, JMessagePanel messageP) {
      super(owner, "About", true);

      this.messageP = messageP;

      Font labelFont = new Font("Dialog", Font.BOLD, 12);

      aboutL = new JLabel("<html>" + ABOUT_INFO + "</html>");
      aboutL.setFont(labelFont);
      aboutL.setBorder(new EmptyBorder(20, 40, 20, 40));

      Container contentPane = getContentPane();

      JPanel southP = new JPanel();
      try {
         southP.add(new JButton( new ProxyAction(this, "dispose", "OK", 'o')));
      } catch(NoSuchMethodException nsme) {
         ImageSystem.LOG.severe(nsme.getMessage());
      }

      contentPane.add(aboutL, BorderLayout.CENTER);
      contentPane.add(southP, BorderLayout.SOUTH);

      setResizable(false);

      pack();

   }

   public void setVersion(String version) {
      StringBuffer builder = new StringBuffer();
      builder.append("<html>")
             .append(ABOUT_INFO)
             .append("<br><br><hr><br>AVRcam")
             .append("<BR>&nbsp;&nbsp;&nbsp;&nbsp;")
             .append(version)
             .append("<html>");
      aboutL.setText(builder.toString());
      pack();
   }

   public void show() {

      setLocationRelativeTo(getOwner());

      if(ImageSystem.DEVICE.isConnected()) {
         try {
            handler = new DataHandler(this);
            ImageSystem.DEVICE.addDataListener(handler);
            messageP.append("Get Version");
            ImageSystem.DEVICE.sendGetVersion();
         } catch(IOException ioe) {
            ImageSystem.LOG.severe("Could not get version: " + ioe.getMessage());
            ImageSystem.DEVICE.removeDataListener(handler);
         }
      }
      super.show();
   }

   public void dispose() {
      ImageSystem.DEVICE.removeDataListener(handler);
      super.dispose();
   }

   private static class DataHandler extends DataAdapter {

      private JAboutDialog dialog;

      public DataHandler(JAboutDialog dialog) {
         this.dialog = dialog;
      }

      public void nck() {
         ImageSystem.DEVICE.removeDataListener(this);
         dialog.setVersion("Unable to retrieve version.");
      }

      public void version(String version) {
         ImageSystem.DEVICE.removeDataListener(this);
         dialog.setVersion(version);
      }

   }

}
