package img.swing;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class JRegisterPanel extends Box {

   public static void main(String[] args) throws Exception {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      int option = new JRegisterPanel().showDialog(null);
      System.out.println(option);
   }

   private static final int UNKNOWN_OPTION = 0x00;
   public static final int OK_OPTION = 0x01;
   public static final int CANCEL_OPTION = 0x02;

   private JDialog dialog;
   private JRegister[] registers;
   private int option;

   public JRegisterPanel() {
      super(BoxLayout.Y_AXIS);

      // the bottom space will be created with a vertical strut
      setBorder(new EmptyBorder(5, 5, 0, 5));

      registers = new JRegister[8];

      for(int i = 0; i < registers.length; i++) {
         registers[i] = new JRegister();
         add(registers[i]);
         add(Box.createVerticalStrut(5));
      }

   }

   public int showDialog(Frame owner) {
      reset();

      option = UNKNOWN_OPTION;
      if(dialog == null) {
         dialog = new JDialog(owner, "Set Registers", true);

         dialog.getContentPane().add(this, BorderLayout.CENTER);
         dialog.getContentPane().add(createButtonPanel(), BorderLayout.SOUTH);
         dialog.pack();
         dialog.setResizable(false);
      }

      dialog.setLocationRelativeTo(owner);
      dialog.show();

      return option;

   }

   public void reset() {
      for(int i = 0; i < registers.length; i++) {
         registers[i].reset();
      }
   }

   public Map getRegisters() {
      Map info = new HashMap();
      for(int i = 0; i < registers.length; i++) {
      JRegister r = registers[i];
         if(r.isChecked()) {
            info.put(r.getRegister(), r.getValue());
         }
      }

      return Collections.unmodifiableMap(info);
   }

   public void ok() {
      option = OK_OPTION;
      dialog.hide();
   }

   public void cancel() {
      option = CANCEL_OPTION;
      dialog.hide();
   }

   private JComponent createButtonPanel() {

      JPanel buttonP = new JPanel(new FlowLayout(FlowLayout.RIGHT));

      buttonP.setBorder(new EtchedBorder());

      try {
         buttonP.add(new JButton(new ProxyAction(this, "ok", "OK", 'o')));
         buttonP.add(new JButton(new ProxyAction(this, "cancel", "Cancel", 'c')));
      } catch(NoSuchMethodException ex) {
      }

      return buttonP;
   }

   private static class JRegister extends JPanel {

      private JCheckBox enableCB;
      private JSpinner registerS;
      private JSpinner valueS;

      public JRegister() {
         super(new FlowLayout(FlowLayout.LEFT));

         try {
            enableCB = new JCheckBox(new ProxyAction(this, "setEnabled", (String)null));
         } catch(Exception e) {
         }

         registerS = new JSpinner(new SpinnerNumberModel(0, 0, 0x90, 1));
         valueS = new JSpinner(new SpinnerNumberModel(0, 0, 0xFF, 1));

         reset();

         add(enableCB);
         add(new JLabel("Register "));
         add(registerS);
         add(new JLabel(" = "));
         add(valueS);
      }

      public void reset() {
         enableCB.setSelected(false);
         registerS.setEnabled(false);
         valueS.setEnabled(false);
         registerS.setValue(new Integer(0));
         valueS.setValue(new Integer(0));
      }

      public boolean isChecked() {
         return enableCB.isSelected();
      }

      public String getRegister() {
         return registerS.getValue().toString();
      }

      public String getValue() {
         return valueS.getValue().toString();
      }

      public void setEnabled() {
         boolean enabled = enableCB.isSelected();
         registerS.setEnabled(enabled);
         valueS.setEnabled(enabled);
      }

   }


}
