package log.util;

import java.io.*;
import java.util.logging.*;
import java.util.zip.*;

/***********************************************************************
 * Defines a File Handler that uses GZIP to compress the data written to
 * the file.
 */
public class GZIPFileHandler extends FileHandler {

   /**
    * Create a GZIP formatted File Handler to compress the size of the
    * log file created.
    * @param pattern The pattern for naming the output file
    * @param limit The maximum number of bytes to write to any one file
    * @param count The number of files to use
    * @param append Specifies append mode
    * @throws IOException If there are IO problems opening the files.
    * @throws java.lang.SecurityException If a security manager exists and
    * if the caller does not have LoggingPermission("control").
    * @throws IllegalArgumentException If limit < 0, or count < 1.
    * @see FileHandler
    */
   public GZIPFileHandler(String pattern,
                          int limit,
                          int count,
                          boolean append) throws IOException,
                                                 SecurityException {
      super(pattern, limit, count, append);
   }

   /**
    * This method overrides its parent's method to inject a GZIPOutputStream.
    * @param out The stream to write the log information to.
    */
   protected void setOutputStream(OutputStream out) {
      try {
         // inject a GZIP Output Stream to write the log information to.
         super.setOutputStream(new GZIPOutputStream(out));
      } catch(Exception e) {
         e.printStackTrace(System.err);
         // fall back to just writing the original stream
         super.setOutputStream(out);
      }
   }

}
