package log.util;

import java.util.logging.*;

import log.swing.table.*;

/***********************************************************************
 * Defines a Handler for the Log to publish the log records to a
 * Table Model.
 */
public class LogHandler extends Handler {

   /**
    * The Table Model to publish the log records to.
    */
   private LogTableModel model;

   /**
    * Create a Log Handler that will publish the log records to a
    * Table Model
    * @param model The table model to publish the log records to.
    * @param level The initial log level this handler will log.
    */
   public LogHandler(LogTableModel model, Level level) {
      super();

      setLevel(level);

      this.model = model;
   }

   /**
    * Publish the given record to the table model.
    * @param record The log record to publish.
    */
   public synchronized void publish(LogRecord record) {

      if(!isLoggable(record)) {
         return;
      }

      // pass the record to the table.
      model.addRecord(record);

   }

   /**
    * Flush the contents of this Handler.  Nothing is done here since
    * the record is automatically displayed in the table.
    */
   public void flush() {
      // nothing needs to be done here
   }

   /**
    * Releases any resources taken up by this Handler.  Nothing is done here
    * since there are no resources allocated.
    * @throws java.lang.SecurityException
    */
   public void close() throws java.lang.SecurityException {
      // nothing needs to be done here
   }

}
