package log.util;

import java.nio.charset.*;
import java.util.logging.*;

/***********************************************************************
 * Defines an XML Formatter with a new XML Header.
 */
public class LogXMLFormatter extends XMLFormatter {

   /**
    * Return the header string for a set of XML formatted records.  This
    * method is overridden to remove the "Standalone" and "DOCTYPE"
    * information.  They are not needed in the Host Software.
    * @param handler The target handler
    * @return The XML header string.
    */
   public String getHead(Handler handler) {

      StringBuffer sb = new StringBuffer();

      sb.append("<?xml version=\"1.0\"");

      String encoding = handler.getEncoding();

      if(encoding == null) {
         // Figure out the default encoding.
         encoding = sun.io.Converters.getDefaultEncodingName();
      }

      // Try to map the encoding name to a canonical name.
      try {
         Charset cs = Charset.forName(encoding);
         encoding = cs.name();
      } catch(Exception ex) {
         // We hit problems finding a canonical name.
         // Just use the raw encoding name.
      }

      sb.append(" encoding=\"")
        .append(encoding)
        .append("\"?>\n<log>\n");

      return sb.toString();

   }

}
